/***************************************************************************
                           dualdrivedialog.cpp
                           -------------------------------
    copyright            : (C) 2010 Guido Scholz
    e-mail               : guido.scholz@bayernline.de
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/***************************************************************************
 This file provides an user interface to change drive confuguration
 ***************************************************************************/

#include <QLabel>
#include <QLayout>
#include <QMessageBox>
#include <QDialogButtonBox>

#include "dualdrivedialog.h"
#include "preferences.h"
#include "resources.h"
#include "srcpmessage.h"

enum {
    MINMMPORT = 0,
    MAXMMPORT = 1,
    MINDCCPORT = 0,
    MAXDCCPORT = 1,
    MINSVPORT = 0,
    MAXSVPORT = 64535, // no limits
    MINSXPORT = 1,
    MAXSXPORT = 8
};



DualDriveDialog::DualDriveDialog(QWidget* parent): QDialog(parent)
{
    setObjectName("DualDriveDialog");
    setWindowTitle(tr("Edit drives"));

    /*Layout to separate OK Cancel buttons from the upper rest*/
    QBoxLayout* baseLayout = new QVBoxLayout(this);
    baseLayout->setSpacing(10);

    /* 1. decoder*/
    /*Layout to separate left and right groupboxes*/
    QBoxLayout* leftRight1Layout = new QHBoxLayout();
    leftRight1Layout->setSpacing(10);
    baseLayout->addLayout(leftRight1Layout);

    /*left groupbox with protocol data*/
    QGroupBox* protocol1GB = new QGroupBox(tr("Protocol"));
    protocol1GB->setObjectName("protocol1GB");
    leftRight1Layout->addWidget(protocol1GB);

    QVBoxLayout* protocol1GBL = new QVBoxLayout();
    protocol1GB->setLayout(protocol1GBL);

    protocol1BG = new QButtonGroup(protocol1GBL);
    protocol1BG->setExclusive(true);

    QRadioButton* mmRB = new QRadioButton(tr("&Maerklin/Motorola"),
            protocol1GB);
    protocol1BG->addButton(mmRB, GaMessage::proMM);
    protocol1GBL->addWidget(mmRB);

    QRadioButton* dccRB = new QRadioButton(tr("&NMRA/DCC"), protocol1GB);
    protocol1BG->addButton(dccRB, GaMessage::proDCC);
    protocol1GBL->addWidget(dccRB);

    QRadioButton* selRB = new QRadioButton(tr("Selectri&x"), protocol1GB);
    protocol1BG->addButton(selRB, GaMessage::proSelectrix);
    protocol1GBL->addWidget(selRB);

    QRadioButton* serverRB = new QRadioButton(tr("Protocol by Ser&ver"),
            protocol1GB);
    protocol1BG->addButton(serverRB, GaMessage::proServer);
    protocol1GBL->addWidget(serverRB);
    protocol1GBL->addStretch();

    connect(protocol1BG, SIGNAL(buttonClicked(int)),
            this, SLOT(slotProtocol1Changed(int)));

    /*right decoder data group box*/
    QGroupBox* decoder1GB = new QGroupBox(tr("Decoder"), this);
    decoder1GB->setObjectName("decoder1GB");
    leftRight1Layout->addWidget(decoder1GB);

    QVBoxLayout* decoder1GBL = new QVBoxLayout();
    decoder1GB->setLayout(decoder1GBL);

    /*line with resettime */
    QHBoxLayout* reset1Layout = new QHBoxLayout();
    reset1Layout->setSpacing(6);
    decoder1GBL->addLayout(reset1Layout);

    QLabel* labelTime = new QLabel(tr("Reset &after (ms):"), decoder1GB);
    reset1Layout->addWidget(labelTime);
    reset1Layout->addStretch();

    activeTime1SB = new QSpinBox(decoder1GB);
    activeTime1SB->setWrapping(true);
    activeTime1SB->setMinimum(50);
    activeTime1SB->setMaximum(2000);
    activeTime1SB->setSingleStep(50);
    reset1Layout->addWidget(activeTime1SB);
    labelTime->setBuddy(activeTime1SB);

    QGridLayout* dec1dataLayout = new QGridLayout();
    dec1dataLayout->setObjectName("dec1dataLayout");
    decoder1GBL->addLayout(dec1dataLayout);

    /*line with srcp bus 1 */
    QLabel* srcpBus1Label = new QLabel(tr("S&RCP-Bus:"), decoder1GB);
    dec1dataLayout->addWidget(srcpBus1Label, 0, 0);
    srcpBus1LE = new QLineEdit(decoder1GB);
    srcpBus1LE->setObjectName("srcpBus1LE");
    srcpBus1LE->setMaxLength(4);
    srcpBus1LE->setMaximumWidth(LEMAXWIDTH);
    dec1dataLayout->addWidget(srcpBus1LE, 0, 1);
    srcpBus1Label->setBuddy(srcpBus1LE);

    /*line with address 1 */
    QLabel* address1Lbl = new QLabel(tr("A&ddress:"), decoder1GB);
    dec1dataLayout->addWidget(address1Lbl, 1, 0);
    address1LE = new QLineEdit(decoder1GB);
    address1LE->setObjectName("address_1");
    dec1dataLayout->addWidget(address1LE, 1, 1);
    address1LE->setMaxLength(4);       // address length of NA protocol
    address1LE->setMaximumWidth(LEMAXWIDTH);
    addressVdt1 = new QIntValidator(0, MAX_GADCC, this);
    address1LE->setValidator(addressVdt1);
    address1Lbl->setBuddy(address1LE);

    /*line with port 1 spinbox */
    port1Label = new QLabel(tr("&Port:"), decoder1GB);
    dec1dataLayout->addWidget(port1Label, 2, 0);

    port1SB = new QSpinBox(decoder1GB);
    port1SB->setObjectName("port1SB");
    port1SB->setWrapping(true);
    port1SB->setMinimum(0);
    port1SB->setMaximum(1);
    dec1dataLayout->addWidget(port1SB, 2, 1);
    port1Label->setBuddy(port1SB);

    xchConn1CB = new QCheckBox(tr("&Exch. conn."), decoder1GB);
    xchConn1CB->setObjectName("xch1");
    dec1dataLayout->addWidget(xchConn1CB, 2, 2);

    /*spacer to push contents of box to top */
    decoder1GBL->addStretch();


    /* 2. decoder*/
    /*Layout to separate left and right groupboxes*/
    QBoxLayout* leftRight2Layout = new QHBoxLayout();
    leftRight2Layout->setSpacing(10);
    baseLayout->addLayout(leftRight2Layout);

    /*left groupbox with protocol data*/
    QGroupBox* protocol2GB = new QGroupBox(tr("Protocol"));
    protocol2GB->setObjectName("protocol2GB");
    leftRight2Layout->addWidget(protocol2GB);

    QVBoxLayout* protocol2GBL = new QVBoxLayout();
    protocol2GB->setLayout(protocol2GBL);

    protocol2BG = new QButtonGroup(protocol2GBL);
    protocol2BG->setExclusive(true);

    QRadioButton* mm2RB = new QRadioButton(tr("&Maerklin/Motorola"),
            protocol2GB);
    protocol2BG->addButton(mm2RB, GaMessage::proMM);
    protocol2GBL->addWidget(mm2RB);

    QRadioButton* dcc2RB = new QRadioButton(tr("&NMRA/DCC"), protocol2GB);
    protocol2BG->addButton(dcc2RB, GaMessage::proDCC);
    protocol2GBL->addWidget(dcc2RB);

    QRadioButton* sel2RB = new QRadioButton(tr("Selectri&x"), protocol2GB);
    protocol2BG->addButton(sel2RB, GaMessage::proSelectrix);
    protocol2GBL->addWidget(sel2RB);

    QRadioButton* server2RB = new QRadioButton(tr("Protocol by Ser&ver"),
            protocol2GB);
    protocol2BG->addButton(server2RB, GaMessage::proServer);
    protocol2GBL->addWidget(server2RB);
    protocol2GBL->addStretch();

    connect(protocol2BG, SIGNAL(buttonClicked(int)),
            this, SLOT(slotProtocol2Changed(int)));

    /*right decoder data group box*/
    QGroupBox* decoder2GB = new QGroupBox(tr("Decoder"), this);
    decoder2GB->setObjectName("decoder2GB");
    leftRight2Layout->addWidget(decoder2GB);

    QVBoxLayout* decoder2GBL = new QVBoxLayout();
    decoder2GB->setLayout(decoder2GBL);

    /*line with resettime */
    QHBoxLayout* reset2Layout = new QHBoxLayout();
    reset2Layout->setSpacing(6);
    decoder2GBL->addLayout(reset2Layout);

    labelTime = new QLabel(tr("Reset &after (ms):"), decoder2GB);
    reset2Layout->addWidget(labelTime);
    reset2Layout->addStretch();

    activeTime2SB = new QSpinBox(decoder2GB);
    activeTime2SB->setWrapping(true);
    activeTime2SB->setMinimum(50);
    activeTime2SB->setMaximum(2000);
    activeTime2SB->setSingleStep(50);
    reset2Layout->addWidget(activeTime2SB);
    labelTime->setBuddy(activeTime2SB);

    QGridLayout* dec2dataLayout = new QGridLayout();
    dec2dataLayout->setObjectName("dec2dataLayout");
    decoder2GBL->addLayout(dec2dataLayout);

    /*line with srcp bus 2 */
    QLabel* srcpBus2Label = new QLabel(tr("S&RCP-Bus:"), decoder2GB);
    dec2dataLayout->addWidget(srcpBus2Label, 0, 0);
    srcpBus2LE = new QLineEdit(decoder2GB);
    srcpBus2LE->setObjectName("srcpBus2LE");
    srcpBus2LE->setMaxLength(4);
    srcpBus2LE->setMaximumWidth(LEMAXWIDTH);
    dec2dataLayout->addWidget(srcpBus2LE, 0, 1);
    srcpBus2Label->setBuddy(srcpBus2LE);

    /*line with address 2 */
    QLabel* address2Lbl = new QLabel(tr("A&ddress:"), decoder2GB);
    dec2dataLayout->addWidget(address2Lbl, 1, 0);
    address2LE = new QLineEdit(decoder2GB);
    address2LE->setObjectName("address_2");
    dec2dataLayout->addWidget(address2LE, 1, 1);
    address2LE->setMaxLength(4);       // address length of NA protocol
    address2LE->setMaximumWidth(LEMAXWIDTH);
    addressVdt2 = new QIntValidator(0, MAX_GADCC, this);
    address2LE->setValidator(addressVdt2);
    address2Lbl->setBuddy(address2LE);

    /*line with port 2 spinbox */
    port2Label = new QLabel(tr("&Port:"), decoder2GB);
    dec2dataLayout->addWidget(port2Label, 2, 0);

    port2SB = new QSpinBox(decoder2GB);
    port2SB->setObjectName("port2SB");
    port2SB->setWrapping(true);
    port2SB->setMinimum(0);
    port2SB->setMaximum(1);
    dec2dataLayout->addWidget(port2SB, 2, 1);
    port2Label->setBuddy(port2SB);

    xchConn2CB = new QCheckBox(tr("&Exch. conn."), decoder2GB);
    xchConn2CB->setObjectName("xch2");
    dec2dataLayout->addWidget(xchConn2CB, 2, 2);

    /*spacer to push contents of box to top */
    decoder2GBL->addStretch();


    QDialogButtonBox* buttonBox = new QDialogButtonBox(
            QDialogButtonBox::Ok |
            QDialogButtonBox::Cancel);

    connect(buttonBox, SIGNAL(accepted()), this, SLOT(validate()));
    connect(buttonBox, SIGNAL(rejected()), this,
            SLOT(reject()));
    baseLayout->addWidget(buttonBox);
}


GaMessage::Protocol DualDriveDialog::getProtocol1()
{
    return (GaMessage::Protocol)protocol1BG->checkedId();
}


GaMessage::Protocol DualDriveDialog::getProtocol2()
{
    return (GaMessage::Protocol)protocol2BG->checkedId();
}


void DualDriveDialog::setProtocol1(GaMessage::Protocol protocol)
{
    QAbstractButton *button;

    if (protocol == GaMessage::proNone) {
        button = protocol1BG->button(pref.protocol);
        if (button != NULL)
            button->setChecked(true);
    }
    else {
        button = protocol1BG->button(protocol);
        if (button != NULL)
            button->setChecked(true);
    }
    updateValidator1();
}


void DualDriveDialog::setProtocol2(GaMessage::Protocol protocol)
{
    QAbstractButton *button;

    if (protocol == GaMessage::proNone) {
        button = protocol2BG->button(pref.protocol);
        if (button != NULL)
            button->setChecked(true);
    }
    else {
        button = protocol2BG->button(protocol);
        if (button != NULL)
            button->setChecked(true);
    }
    updateValidator2();
}


void DualDriveDialog::setActiveTime1(int atime)
{
    if (atime != -1)
        activeTime1SB->setValue(atime);
    else
        activeTime1SB->setValue(pref.activetime);
}


void DualDriveDialog::setActiveTime2(int atime)
{
    if (atime != -1)
        activeTime2SB->setValue(atime);
    else
        activeTime2SB->setValue(pref.activetime);
}


int DualDriveDialog::getActiveTime1()
{
    return activeTime1SB->value();
}


int DualDriveDialog::getActiveTime2()
{
    return activeTime2SB->value();
}


int DualDriveDialog::getSRCPBus1()
{
    return srcpBus1LE->text().toInt();
}


int DualDriveDialog::getSRCPBus2()
{
    return srcpBus2LE->text().toInt();
}


void DualDriveDialog::setSRCPBus1(int bus)
{
    srcpBus1LE->setText(QString::number(bus));
}


void DualDriveDialog::setSRCPBus2(int bus)
{
    srcpBus2LE->setText(QString::number(bus));
}


int DualDriveDialog::getAddress1()
{
    return address1LE->text().toInt();
}


int DualDriveDialog::getAddress2()
{
    return address2LE->text().toInt();
}


void DualDriveDialog::setAddress1(int addr)
{
    address1LE->setText(QString::number(addr));
}


void DualDriveDialog::setAddress2(int addr)
{
    address2LE->setText(QString::number(addr));
}


void DualDriveDialog::setPort1(int aport)
{
    port1SB->setValue(aport);
}


void DualDriveDialog::setPort2(int aport)
{
    port2SB->setValue(aport);
}


int DualDriveDialog::getPort1()
{
    return port1SB->value();
}


int DualDriveDialog::getPort2()
{
    return port2SB->value();
}


int DualDriveDialog::getXChangeConn1()
{
    return xchConn1CB->isEnabled() ?
        (xchConn1CB->isChecked() ? 1 : 0) : -1;
}


int DualDriveDialog::getXChangeConn2()
{
    return xchConn2CB->isEnabled() ?
        (xchConn2CB->isChecked() ? 1 : 0) : -1;
}


void DualDriveDialog::setXChangeConn1(int xch)
{
    if (xch == -1)
        xchConn1CB->setChecked(false);
    else
        xchConn1CB->setChecked(xch);
}

void DualDriveDialog::setXChangeConn2(int xch)
{
    if (xch == -1)
        xchConn2CB->setChecked(false);
    else
        xchConn2CB->setChecked(xch);
}

void DualDriveDialog::slotProtocol1Changed(int)
{
    updateValidator1();
}


void DualDriveDialog::slotProtocol2Changed(int)
{
    updateValidator2();
}


void DualDriveDialog::updateValidator1()
{
    /*
     *  id                          protocol
     *  -------------------------------------
     *  -1                          none
     *   GaMessage::proMM           MM
     *   GaMessage::proDCC          DCC
     *   GaMessage::proSelectrix    Selectrix
     *   GaMessage::proServer       Server
     *  -------------------------------------
     */

    int prot = protocol1BG->checkedId();

    switch (prot) {
        case GaMessage::proMM:
            // MM
            addressVdt1->setTop(MAX_GAMM);
            updateAddressTooltip1();

            port1Label->setEnabled(false);
            port1SB->setEnabled(false);
            // new range for port spinboxes
            port1SB->setMinimum(MINMMPORT);
            port1SB->setMaximum(MAXMMPORT);

            xchConn1CB->setEnabled(true);

            if (address1LE->text().toInt() > MAX_GAMM)
                address1LE->setText(QString::number(MAX_GAMM));

            break;

        case GaMessage::proDCC:
            //DCC
            addressVdt1->setTop(MAX_GADCC);
            updateAddressTooltip1();

            port1Label->setEnabled(false);
            port1SB->setEnabled(false);
            // new range for port spinboxes
            port1SB->setMinimum(MINDCCPORT);
            port1SB->setMaximum(MAXDCCPORT);

            xchConn1CB->setEnabled(true);

            break;

        case GaMessage::proSelectrix:
            //Selectrix
            addressVdt1->setTop(MAX_GASX);
            updateAddressTooltip1();

            // new range for port spinboxes
            port1SB->setMinimum(MINSXPORT);
            port1SB->setMaximum(MAXSXPORT);

            //xchConn1CB->setEnabled(false);

            port1Label->setEnabled(true);
            port1SB->setEnabled(true);

            break;

        case 3:
            // Server
            // MAGIC: Validator limits for Protocol by Server
            addressVdt1->setTop(9999);
            updateAddressTooltip1();

            // new range for port spinboxes
            port1SB->setMinimum(MINSVPORT);
            port1SB->setMaximum(MAXSVPORT);

            //xchConn1CB->setEnabled(false);

            port1Label->setEnabled(true);
            port1SB->setEnabled(true);

            break;

        default:
            // undefined protocol
            break;
    }
}

void DualDriveDialog::updateValidator2()
{
    /*
     *  id  protocol
     *  -------------
     *  -1  none
     *   0  MM
     *   1  DCC
     *   2  Selectrix
     *   3  Server
     *  -------------
     */

    int prot = protocol2BG->checkedId();

    switch (prot) {
        case 0:
            // MM
            addressVdt2->setTop(MAX_GAMM);
            updateAddressTooltip2();

            port2Label->setEnabled(false);
            port2SB->setEnabled(false);
            // new range for port spinboxes
            port2SB->setMinimum(MINMMPORT);
            port2SB->setMaximum(MAXMMPORT);

            xchConn2CB->setEnabled(true);

            if (address2LE->text().toInt() > MAX_GAMM)
                address2LE->setText(QString::number(MAX_GAMM));

            break;

        case 1:
            //DCC
            addressVdt2->setTop(MAX_GADCC);
            updateAddressTooltip2();

            port2Label->setEnabled(false);
            port2SB->setEnabled(false);
            // new range for port spinboxes
            port2SB->setMinimum(MINDCCPORT);
            port2SB->setMaximum(MAXDCCPORT);

            xchConn2CB->setEnabled(true);

            break;

        case 2:
            //Selectrix
            addressVdt2->setTop(MAX_GASX);
            updateAddressTooltip2();

            // new range for port spinboxes
            port2SB->setMinimum(MINSXPORT);
            port2SB->setMaximum(MAXSXPORT);

            //xchConn2CB->setEnabled(false);

            port2Label->setEnabled(true);
            port2SB->setEnabled(true);

            break;

        case 3:
            // Server
            // MAGIC: Validator limits for Protocol by Server
            addressVdt2->setTop(9999);
            updateAddressTooltip2();

            // new range for port spinboxes
            port2SB->setMinimum(MINSVPORT);
            port2SB->setMaximum(MAXSVPORT);

            //xchConn2CB->setEnabled(false);

            port2Label->setEnabled(true);
            port2SB->setEnabled(true);

            break;

        default:
            // undefined protocol
            break;
    }
}

void DualDriveDialog::updateAddressTooltip1()
{
    address1LE->setToolTip(tr(
                "Enter decoder address.\n"
                "Valid range is %1..%2.")
            .arg(addressVdt1->bottom())
            .arg(addressVdt1->top()));
}


void DualDriveDialog::updateAddressTooltip2()
{
    address2LE->setToolTip(tr(
                "Enter decoder address.\n"
                "Valid range is %1..%2.")
            .arg(addressVdt2->bottom())
            .arg(addressVdt2->top()));
}


void DualDriveDialog::validate()
{
    if (address1LE->isEnabled() && !address1LE->hasAcceptableInput()) {
        address1LE->setFocus();
        address1LE->selectAll();
        QMessageBox::warning(this, tr("Unvalid address detected"),
                tr("Value of decoder address is not valid.\n"
                    "Valid range is %1..%2.")
                .arg(addressVdt1->bottom())
                .arg(addressVdt1->top())
        , QMessageBox::Ok);
        return;
    }

    if (address2LE->isEnabled() && !address2LE->hasAcceptableInput()) {
        address2LE->setFocus();
        address2LE->selectAll();
        QMessageBox::warning(this, tr("Unvalid address detected"),
                tr("Value of decoder address is not valid.\n"
                    "Valid range is %1..%2.")
                .arg(addressVdt2->bottom())
                .arg(addressVdt2->top())
        , QMessageBox::Ok);
        return;
    }
    accept();
}

