/***************************************************************************
                           finder.cpp
                           version 0.5.2 $Revision: 1.14 $
                           -------------------------------
    copyright            : (C) 1999-2003 by Stefan Preis
                         : (C) 2004-2007 by Guido Scholz
    email                : guido.scholz@bayernline.de
    last modified        : $Date: 2007/10/10 19:41:51 $
***************************************************************************/

/*****************************************************************************
 *                                                                            *
 *   This program is free software; you can redistribute it and/or modify     *
 *   it under the terms of the GNU General Public License as published by     *
 *   the Free Software Foundation; either version 2 of the License, or        *
 *   (at your option) any later version.                                      *
 *                                                                            *
 ******************************************************************************/
/******************************************************************************
   this code shows a window where user enters an address to be searched for
 ******************************************************************************/

#include <QLayout>
#include <QLabel>
#include <QGroupBox>

#include "finder.h"
#include "preferences.h"


Finder::Finder(QWidget* parent): QDialog(parent)
{
    setWindowTitle(tr("Element locator"));
    /*Layout to separate Search and Cancel buttons form the upper rest*/
    QVBoxLayout* baseLayout = new QVBoxLayout();
    baseLayout->setSpacing(10);
    setLayout(baseLayout);
    
    /*line with search editline*/
    QHBoxLayout* searchLayout = new QHBoxLayout();
    searchLayout->setSpacing(6);
    baseLayout->addLayout(searchLayout);

    QLabel *title = new QLabel(tr("&Find:"));
    searchLayout->addWidget(title);
    searchLayout->addStretch();
    leSearch = new QLineEdit();
    searchLayout->addWidget(leSearch);
    title->setBuddy(leSearch);
    leSearch->setMaxLength(10);
    // if at least one char has been entered highlight search button
    connect(leSearch, SIGNAL(textChanged(const QString&)),
            this, SLOT(slotActivateSearchButt(const QString&)));

    /*group box to choose data field*/
    QGroupBox* dataGB = new QGroupBox(tr("Data fields"));
    baseLayout->addWidget(dataGB);
    QVBoxLayout* dataL = new QVBoxLayout();
    dataL->setSpacing(6);
    dataGB->setLayout(dataL);
    dataBG = new QButtonGroup(dataGB);
    dataBG->setExclusive(true);
    QRadioButton *rbSearchText =
        new QRadioButton(tr("&Text fields"), dataGB);
    dataBG->addButton(rbSearchText);
    dataBG->setId(rbSearchText, SRCH_TX);
    dataL->addWidget(rbSearchText);

    QRadioButton* rb = new QRadioButton(tr("Decoder address &1"), dataGB);
    dataBG->addButton(rb);
    dataBG->setId(rb, SRCH_A1);
    dataL->addWidget(rb);

    rb = new QRadioButton(tr("Decoder address &2"), dataGB);
    dataBG->addButton(rb);
    dataBG->setId(rb, SRCH_A2);
    dataL->addWidget(rb);
    
    rbSearchText->setChecked(true);

    /*group box to choose find count*/
    QGroupBox* matchGB = new QGroupBox(tr("Match counter"));
    baseLayout->addWidget(matchGB);
    QVBoxLayout* matchL = new QVBoxLayout();
    matchL->setSpacing(6);
    matchGB->setLayout(matchL);
    matchBG = new QButtonGroup(matchGB);
    matchBG->setExclusive(true);

    QRadioButton *rbSearchSingle =
        new QRadioButton(tr("&First/only one match"), matchGB);
    matchBG->addButton(rbSearchSingle);
    matchBG->setId(rbSearchSingle, SRCH_SINGLE);
    matchL->addWidget(rbSearchSingle);

    rb = new QRadioButton(tr("&All matches"), matchGB);
    matchBG->addButton(rb);
    matchBG->setId(rb, SRCH_MULTI);
    matchL->addWidget(rb);
    
    rbSearchSingle->setChecked(true);

    /*separated line with Search and Cancel buttons*/
    QHBoxLayout* buttonLayout = new QHBoxLayout();
    buttonLayout->setSpacing(6);
    baseLayout->addLayout(buttonLayout);
    buttonLayout->addStretch();
    buttSearch = new QPushButton(tr("Search"), this);
    buttonLayout->addWidget(buttSearch);
    buttSearch->setDefault(true);
    buttSearch->setEnabled(false);
    connect(buttSearch, SIGNAL(clicked()), this, SLOT(accept()));
    QPushButton* buttCancel = new QPushButton(tr("Cancel"), this);
    buttonLayout->addWidget(buttCancel);
    connect(buttCancel, SIGNAL(clicked()), this, SLOT(reject()));
    buttSearch->setToolTip(
            tr("Press this button to start search"));
    buttCancel->setToolTip(
            tr("Press this button to close this window"));
}


QString Finder::getSearchText()
{
    return leSearch->text();
}


int Finder::getDataType()
{
    return dataBG->checkedId();
}


int Finder::getMatchType()
{
    return matchBG->checkedId();
}


void Finder::slotActivateSearchButt(const QString&)
{
    // only enable search button if at least one char has been entered
    buttSearch->setEnabled(!leSearch->text().isEmpty());
}
