/***************************************************************************
                           routedialog.cpp
                           -------------------------------
    copyright            : (C) 2005-2008 Guido Scholz
    email                : guido-scholz@gmx.net
    last modified        : $Date: 2009-10-27 20:29:24 $
                           $Revision: 1.53 $
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/***************************************************************************
   this file provides a small user interface to select a certain route
   to be activated
 ***************************************************************************/

#include <QLayout>
#include <QGridLayout>
#include <QRadioButton>
#include <QDialogButtonBox>
#include <QHeaderView>

#include "element.h"
#include "preferences.h"
#include "resources.h"
#include "routedialog.h"
#include "routeelementdialog.h"
#include "routeelementlvi.h"



RouteDialog::RouteDialog(QWidget* parent): QDialog(parent),
    startSignalElPtr(NULL), stopSignalElPtr(NULL)
{
    setObjectName("EditRouteDialog");
    setWindowTitle(tr("Edit route"));
    tabWidget = new QTabWidget(this);

    // Validator for all bus input lines
    busValidator = new QIntValidator(1, 999, this);
    // Validator for train ids
    trainidValidator = new QIntValidator(0, 999999, this);
    // Validator for track numbers
    tracknumberValidator = new QIntValidator(1, 99999, this);


    addIdentificationTab();
    addElementsTab();
    addRouteTypeTab();
    addAutomaticTab();

    QDialogButtonBox* buttonBox = new QDialogButtonBox(
            QDialogButtonBox::Ok | QDialogButtonBox::Cancel);

    connect(buttonBox, SIGNAL(accepted()), this, SLOT(accept()));
    connect(buttonBox, SIGNAL(rejected()), this, SLOT(reject()));

    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->addWidget(tabWidget);
    mainLayout->addWidget(buttonBox);
    setLayout(mainLayout);
}

/*
 * add tab for route identification data
 */
void RouteDialog::addIdentificationTab()
{
    QWidget *w = new QWidget();
    /*vertical layout for identificationGB and an other horiz. layout*/
    QVBoxLayout* tabLayout = new QVBoxLayout();
    w->setLayout(tabLayout);

    QGroupBox* identificationGB = new QGroupBox(tr("Identification"));
    QGridLayout *identificationGBL = new QGridLayout();
    identificationGB->setLayout(identificationGBL);
    tabLayout->addWidget(identificationGB);

    /*line with route name*/
    QLabel* lblRouteName = new QLabel(tr("Na&me"));
    identificationGBL->addWidget(lblRouteName, 0 , 0);
    routeNameLE = new QLineEdit();
    lblRouteName->setBuddy(routeNameLE);
    identificationGBL->addWidget(routeNameLE, 0, 1, 1, 2);

    /*line with track number (Gleisnummer)*/
    QLabel* lblRouteNumber = new QLabel(tr("Identification n&umber"));
    identificationGBL->addWidget(lblRouteNumber, 1, 0);
    routeNumberLE = new QLineEdit();
    routeNumberLE->setMaximumWidth(LEMAXWIDTH);
    routeNumberLE->setValidator(tracknumberValidator);
    lblRouteNumber->setBuddy(routeNumberLE);
    identificationGBL->addWidget(routeNumberLE, 1, 1);
    routeNumberLE->setToolTip(tr(
                "Enter an unique identification number for\n"
                "this route. The valid range is %1 to %2.")
            .arg(tracknumberValidator->bottom())
            .arg(tracknumberValidator->top()));

    /*line with train number*/
    QLabel* lblRouteTrain = new QLabel(tr("&Train number"));
    identificationGBL->addWidget(lblRouteTrain, 2, 0);
    routeTrainLE = new QLineEdit();
    routeTrainLE->setMaximumWidth(LEMAXWIDTH);
    routeTrainLE->setValidator(trainidValidator);
    lblRouteTrain->setBuddy(routeTrainLE);
    identificationGBL->addWidget(routeTrainLE, 2, 1);
    routeTrainLE->setToolTip(tr(
                "Enter an unique identification number for\n"
                "the train, currently occupying this route.\n"
                "The valid range is %1 to %2.")
            .arg(trainidValidator->bottom())
            .arg(trainidValidator->top()));

    /*line with route length*/
    QLabel* lblRouteLength = new QLabel(tr("&Length [mm]"));
    identificationGBL->addWidget(lblRouteLength, 3, 0);
    routeLengthLE = new QLineEdit();
    routeLengthLE->setMaximumWidth(LEMAXWIDTH);
    lblRouteLength->setBuddy(routeLengthLE);
    identificationGBL->addWidget(routeLengthLE, 3, 1);
    routeLengthLE->setToolTip(tr(
                "Enter the actual lenght of the route in mm."));

    /*line with route speedlimit*/
    QLabel* lblRouteSpeedLimit = new QLabel(tr("&Speedlimit [km/h]"));
    identificationGBL->addWidget(lblRouteSpeedLimit, 4, 0);
    routeSpeedLimitLE = new QLineEdit();
    routeSpeedLimitLE->setMaximumWidth(LEMAXWIDTH);
    lblRouteSpeedLimit->setBuddy(routeSpeedLimitLE);
    identificationGBL->addWidget(routeSpeedLimitLE, 4, 1);
    routeSpeedLimitLE->setToolTip(tr(
                "Enter speedlimit of route in km/h."));

    /*horizontal layout for start and stop signal data group boxes*/
    QHBoxLayout* signalStaStoLayout = new QHBoxLayout();
    tabLayout->addLayout(signalStaStoLayout);

    /*start signal group box*/
    QGroupBox* startsignalGB = new QGroupBox(tr("Start signal"));
    QVBoxLayout* startSigGBLayout = new QVBoxLayout();
    startsignalGB->setLayout(startSigGBLayout);
    signalStaStoLayout->addWidget(startsignalGB);

    /*line with start signal name*/
    QHBoxLayout* startSignalLayout = new QHBoxLayout();
    startSigGBLayout->addLayout(startSignalLayout);

    QLabel* startSignalNameLB = new QLabel(tr("Name"));
    startSignalLayout->addWidget(startSignalNameLB);
    startSignalLayout->addStretch();
    startSignalNameLE = new QLineEdit();
    startSignalNameLE->setReadOnly(true);
    startSignalNameLE->setFocusPolicy(Qt::NoFocus);
    startSignalNameLE->setMaximumWidth(LEMAXWIDTH);
    startSignalLayout->addWidget(startSignalNameLE);

    /*Validator for entry and exit track number range*/
    QIntValidator* trackValidator = new QIntValidator(0, 999, this);

    /*line with entry track number*/
    QHBoxLayout* entryTrackLayout = new QHBoxLayout();
    startSigGBLayout->addLayout(entryTrackLayout);

    QLabel* entryTrackNumberLB = new QLabel(tr("&Entry track"));
    entryTrackLayout->addWidget(entryTrackNumberLB);
    entryTrackLayout->addStretch();
    entryTrackNumberLE = new QLineEdit();
    entryTrackNumberLB->setBuddy(entryTrackNumberLE);
    entryTrackNumberLE->setMaximumWidth(LEMAXWIDTH);
    entryTrackNumberLE->setValidator(trackValidator);
    entryTrackLayout->addWidget(entryTrackNumberLE);
    entryTrackNumberLE->setToolTip(tr(
                "Enter an unique identification number for\n"
                "this track. The valid range is %1 to %2.")
            .arg(trackValidator->bottom())
            .arg(trackValidator->top()));

    /*line with start signal SRCP bus*/
    QHBoxLayout* startSigSrcpBusLayout = new QHBoxLayout();
    startSigGBLayout->addLayout(startSigSrcpBusLayout);

    QLabel* startSignalSrcpBusLB = new QLabel(tr("SRC&P-Bus"));
    startSigSrcpBusLayout->addWidget(startSignalSrcpBusLB);
    startSigSrcpBusLayout->addStretch();
    startSignalSrcpBusLE = new QLineEdit();
    startSignalSrcpBusLE->setMaximumWidth(LEMAXWIDTH);
    startSignalSrcpBusLE->setValidator(busValidator);
    startSignalSrcpBusLB->setBuddy(startSignalSrcpBusLE);
    startSigSrcpBusLayout->addWidget(startSignalSrcpBusLE);
    connect(startSignalSrcpBusLE, SIGNAL(textChanged(const QString&)),
            this, SLOT(startSignalBusChanged(const QString&)));

    /*line with start signal adress*/
    QHBoxLayout* startSigAddrLayout = new QHBoxLayout();
    startSigGBLayout->addLayout(startSigAddrLayout);

    QLabel* startSignalAddressLB = new QLabel(tr("&Address"));
    startSigAddrLayout->addWidget(startSignalAddressLB);
    startSigAddrLayout->addStretch();
    startSignalAddressLE = new QLineEdit();
    startSignalAddressLE->setMaximumWidth(LEMAXWIDTH);
    startSignalAddressLE->setMaxLength(4);
    startSignalAddressLB->setBuddy(startSignalAddressLE);
    startSigAddrLayout->addWidget(startSignalAddressLE);
    connect(startSignalAddressLE, SIGNAL(textChanged(const QString&)),
            this, SLOT(startSignalAddressChanged(const QString&)));

    /*line with start signal state*/
    QHBoxLayout* startSigStateLayout = new QHBoxLayout();
    startSigGBLayout->addLayout(startSigStateLayout);

    QLabel* startSignalStateLB = new QLabel(tr("S&tate"));
    startSigStateLayout->addWidget(startSignalStateLB);
    startSigStateLayout->addStretch();
    startSignalStateSB = new QSpinBox();
    startSignalStateSB->setMinimum(0);
    startSignalStateSB->setMaximum(3);
    startSignalStateLB->setBuddy(startSignalStateSB);
    startSigStateLayout->addWidget(startSignalStateSB);

    startSigGBLayout->addStretch();


    /*stop signal group box*/
    QGroupBox* stopSignalGB = new QGroupBox(tr("Stop signal"));
    signalStaStoLayout->addWidget(stopSignalGB);
    QVBoxLayout* stopSigGBLayout = new QVBoxLayout();
    stopSignalGB->setLayout(stopSigGBLayout);

    /*line with stop signal name*/
    QHBoxLayout* stopSignalLayout = new QHBoxLayout();
    stopSigGBLayout->addLayout(stopSignalLayout);

    QLabel* stopSignalNameLB = new QLabel(tr("Name"));
    stopSignalLayout->addWidget(stopSignalNameLB);
    stopSignalLayout->addStretch();
    stopSignalNameLE = new QLineEdit();
    stopSignalNameLE->setMaximumWidth(LEMAXWIDTH);
    stopSignalNameLE->setReadOnly(true);
    stopSignalNameLE->setFocusPolicy(Qt::NoFocus);
    stopSignalLayout->addWidget(stopSignalNameLE);

    /*line with exit track number*/
    QHBoxLayout* exitTrackLayout = new QHBoxLayout();
    stopSigGBLayout->addLayout(exitTrackLayout);

    QLabel* exitTrackNumberLB = new QLabel(tr("E&xit track"));
    exitTrackLayout->addWidget(exitTrackNumberLB);
    exitTrackLayout->addStretch();
    exitTrackNumberLE = new QLineEdit();
    exitTrackNumberLB->setBuddy(exitTrackNumberLE);
    exitTrackNumberLE->setMaximumWidth(LEMAXWIDTH);
    exitTrackNumberLE->setValidator(trackValidator);
    exitTrackLayout->addWidget(exitTrackNumberLE);
    exitTrackNumberLE->setToolTip(tr(
                "Enter an unique identification number for\n"
                "this track. The valid range is %1 to %2.")
            .arg(trackValidator->bottom())
            .arg(trackValidator->top()));

    /*line with stop signal SRCP bus*/
    QHBoxLayout* stopSigSrcpBusLayout = new QHBoxLayout();
    stopSigGBLayout->addLayout(stopSigSrcpBusLayout);

    QLabel* stopSignalSrcpBusLB = new QLabel(tr("SRCP-&Bus"));
    stopSigSrcpBusLayout->addWidget(stopSignalSrcpBusLB);
    stopSigSrcpBusLayout->addStretch();
    stopSignalSrcpBusLE = new QLineEdit();
    stopSignalSrcpBusLE->setMaximumWidth(LEMAXWIDTH);
    stopSignalSrcpBusLE->setValidator(busValidator);
    stopSignalSrcpBusLB->setBuddy(stopSignalSrcpBusLE);
    stopSigSrcpBusLayout->addWidget(stopSignalSrcpBusLE);
    connect(stopSignalSrcpBusLE, SIGNAL(textChanged(const QString&)),
            this, SLOT(stopSignalBusChanged(const QString&)));

    /*line with stop signal adress*/
    QHBoxLayout* stopSigAddrLayout = new QHBoxLayout();
    stopSigGBLayout->addLayout(stopSigAddrLayout);

    QLabel* stopSignalAddressLB = new QLabel(tr("Add&ress"));
    stopSigAddrLayout->addWidget(stopSignalAddressLB);
    stopSigAddrLayout->addStretch();
    stopSignalAddressLE = new QLineEdit();
    stopSignalAddressLE->setMaximumWidth(LEMAXWIDTH);
    stopSignalAddressLE->setMaxLength(4);
    stopSignalAddressLB->setBuddy(stopSignalAddressLE);
    stopSigAddrLayout->addWidget(stopSignalAddressLE);
    connect(stopSignalAddressLE, SIGNAL(textChanged(const QString&)),
            this, SLOT(stopSignalAddressChanged(const QString&)));

    stopSigGBLayout->addStretch();

    // push group boxes to top
    tabLayout->addStretch();

    tabWidget->addTab(w, tr("&Identification"));
}

/*
 * add tab with options for automatic route switching
 */
void RouteDialog::addAutomaticTab()
{
    QWidget *w = new QWidget();
    /*horizontal layout for route activation and release group boxes*/
    QHBoxLayout* tabLayout = new QHBoxLayout();
    w->setLayout(tabLayout);

    /*activate route group box*/
    QGroupBox* activateGB = new QGroupBox(tr("Activate route"));
    tabLayout->addWidget(activateGB);
    QVBoxLayout* activateGBL = new QVBoxLayout(activateGB);
    activateGB->setLayout(activateGBL);

    /*line with CheckBox to activate FB switching*/
    activatefbCB = new QCheckBox(tr("&Enable feedback activation"),
            activateGB);
    activateGBL->addWidget(activatefbCB);

    /*two lines with radio buttons to choose feedback signal direction*/
    activateRouteGB = new QGroupBox(tr("Switch on feedback response"));
    activateGBL->addWidget(activateRouteGB);
    QVBoxLayout* activateRouteGBL = new QVBoxLayout();
    activateRouteGB->setLayout(activateRouteGBL);

    activateRouteBG = new QButtonGroup(activateRouteGB);
    activateRouteBG->setExclusive(true);

    QRadioButton* activateOnActivationRB = new QRadioButton(
            tr("Ac&tivation"));
    activateRouteGBL->addWidget(activateOnActivationRB);
    activateRouteBG->addButton(activateOnActivationRB, 0);

    QRadioButton* activateOnDeactivationRB = new QRadioButton(
            tr("&Deactivation"));
    activateRouteGBL->addWidget(activateOnDeactivationRB);
    activateRouteBG->addButton(activateOnDeactivationRB, 1);

    /*line with SRCP bus for activation by feedback*/
    QHBoxLayout* activateSrcpBusLayout = new QHBoxLayout();
    activateGBL->addLayout(activateSrcpBusLayout);

    activateSrcpBusLB = new QLabel(tr("B&us (s88/SRCP)"));
    activateSrcpBusLayout->addWidget(activateSrcpBusLB);
    activateSrcpBusLayout->addStretch();
    activateSrcpBusLE = new QLineEdit();
    activateSrcpBusLE->setMaximumWidth(LEMAXWIDTH);
    activateSrcpBusLE->setValidator(busValidator);
    activateSrcpBusLB->setBuddy(activateSrcpBusLE);
    activateSrcpBusLayout->addWidget(activateSrcpBusLE);

    /*line with contact for activation by feedback*/
    QHBoxLayout* activateContactLayout = new QHBoxLayout();
    activateGBL->addLayout(activateContactLayout);

    activateContactLB = new QLabel(tr("C&ontact (1 - 496)"),
            activateGB);
    activateContactLayout->addWidget(activateContactLB);
    activateContactLayout->addStretch();
    activateContactSB = new QSpinBox();
    activateContactSB->setMinimum(1);
    activateContactSB->setMaximum(496);
    activateContactLB->setBuddy(activateContactSB);
    activateContactLayout->addWidget(activateContactSB);

    /*line with module for activation by feedback*/
    QHBoxLayout* activateModuleLayout = new QHBoxLayout();
    activateGBL->addLayout(activateModuleLayout);

    activateModuleLB = new QLabel(tr("Module (1 - %1)")
            .arg(pref.fbfactor == 0 ? 31 : 62));
    activateModuleLayout->addWidget(activateModuleLB);
    activateModuleLayout->addStretch();
    activateModuleLE = new QLineEdit();
    activateModuleLE->setMaximumWidth(LEMAXWIDTH);
    activateModuleLE->setFocusPolicy(Qt::NoFocus);
    //activateModuleLB->setBuddy(activateModuleLE);
    activateModuleLayout->addWidget(activateModuleLE);

    /*line with port for activation by feedback*/
    QHBoxLayout* activatePortLayout = new QHBoxLayout();
    activateGBL->addLayout(activatePortLayout);

    activatePortLB = new QLabel(tr("Port (1 - %1)")
            .arg(pref.fbfactor == 0 ? 16 : 8));
    activatePortLayout->addWidget(activatePortLB);
    activatePortLayout->addStretch();
    activatePortLE = new QLineEdit();
    activatePortLE->setMaximumWidth(LEMAXWIDTH);
    activatePortLE->setFocusPolicy(Qt::NoFocus);
    //activatePortLB->setBuddy(activatePortLE);
    activatePortLayout->addWidget(activatePortLE);

    // push items to top
    activateGBL->addStretch();


    /*release route group box*/
    QGroupBox* releaseGB = new QGroupBox(tr("Release route"));
    tabLayout->addWidget(releaseGB);
    QVBoxLayout* releaseGBL = new QVBoxLayout();
    releaseGB->setLayout(releaseGBL);

    /*line with CheckBox to release FB switching*/
    releasefbCB = new QCheckBox(tr("Enable &feedback release"),
            releaseGB);
    releaseGBL->addWidget(releasefbCB);

    /*two lines with radio buttons to choose feedback signal direction*/
    releaseRouteGB = new QGroupBox(
            tr("Switch on feedback response"));
    releaseGBL->addWidget(releaseRouteGB);
    QVBoxLayout* releaseFBL = new QVBoxLayout();
    releaseRouteGB->setLayout(releaseFBL);

    releaseRouteBG = new QButtonGroup(releaseRouteGB);
    releaseRouteBG->setExclusive(true);

    QRadioButton* releaseOnActivationRB = new QRadioButton(
            tr("Ac&tivation"));
    releaseFBL->addWidget(releaseOnActivationRB);
    releaseRouteBG->addButton(releaseOnActivationRB, 0);

    QRadioButton* releaseOnDeactivationRB = new QRadioButton(
            tr("&Deactivation"));
    releaseFBL->addWidget(releaseOnDeactivationRB);
    releaseRouteBG->addButton(releaseOnDeactivationRB, 1);

    /*line with SRCP bus for activation by feedback*/
    QHBoxLayout* releaseSrcpBusLayout = new QHBoxLayout();
    releaseGBL->addLayout(releaseSrcpBusLayout);

    releaseSrcpBusLB = new QLabel(tr("Bus (s&88/SRCP)"));
    releaseSrcpBusLayout->addWidget(releaseSrcpBusLB);
    releaseSrcpBusLayout->addStretch();
    releaseSrcpBusLE = new QLineEdit();
    releaseSrcpBusLE->setMaximumWidth(LEMAXWIDTH);
    releaseSrcpBusLE->setValidator(busValidator);
    releaseSrcpBusLB->setBuddy(releaseSrcpBusLE);
    releaseSrcpBusLayout->addWidget(releaseSrcpBusLE);

    /*line with contact for activation by feedback*/
    QHBoxLayout* releaseContactLayout = new QHBoxLayout();
    releaseGBL->addLayout(releaseContactLayout);

    releaseContactLB = new QLabel(tr("&Contact (1 - 496)"));
    releaseContactLayout->addWidget(releaseContactLB);
    releaseContactLayout->addStretch();
    releaseContactSB = new QSpinBox();
    releaseContactSB->setMinimum(1);
    releaseContactSB->setMaximum(496);
    releaseContactLB->setBuddy(releaseContactSB);
    releaseContactLayout->addWidget(releaseContactSB);

    /*line with module for activation by feedback*/
    QHBoxLayout* releaseModuleLayout = new QHBoxLayout();
    releaseGBL->addLayout(releaseModuleLayout);

    releaseModuleLB = new QLabel(tr("Module (1 - %1)")
            .arg(pref.fbfactor == 0 ? 31 : 62));
    releaseModuleLayout->addWidget(releaseModuleLB);
    releaseModuleLayout->addStretch();
    releaseModuleLE = new QLineEdit();
    releaseModuleLE->setMaximumWidth(LEMAXWIDTH);
    releaseModuleLE->setFocusPolicy(Qt::NoFocus);
    //releaseModuleLB->setBuddy(releaseModuleLE);
    releaseModuleLayout->addWidget(releaseModuleLE);

    /*line with port activation by feedback*/
    QHBoxLayout* releasePortLayout = new QHBoxLayout();
    releaseGBL->addLayout(releasePortLayout);

    releasePortLB = new QLabel(tr("Port (1 - %1)")
            .arg(pref.fbfactor == 0 ? 16 : 8));
    releasePortLayout->addWidget(releasePortLB);
    releasePortLayout->addStretch();
    releasePortLE = new QLineEdit();
    releasePortLE->setMaximumWidth(LEMAXWIDTH);
    releasePortLE->setFocusPolicy(Qt::NoFocus);
    //releasePortLB->setBuddy(releasePortLE);
    releasePortLayout->addWidget(releasePortLE);

    // push items to top
    releaseGBL->addStretch();

    connect(activatefbCB, SIGNAL(toggled(bool)),
            this, SLOT(activateCBchanged(bool)));
    connect(releasefbCB, SIGNAL(toggled(bool)),
            this, SLOT(releaseCBchanged(bool)));
    connect(activateContactSB, SIGNAL(valueChanged(int)),
            this, SLOT(activateContactSBChanged(int)));
    connect(releaseContactSB, SIGNAL(valueChanged(int)),
            this, SLOT(releaseContactSBChanged(int)));

    tabWidget->addTab(w, tr("&Automatic"));
}

/*
 * add tab with route type group box
 */
void RouteDialog::addRouteTypeTab()
{
    QWidget *w = new QWidget();
    /*vertical layout for identificationGB and an other horiz. layout*/
    QVBoxLayout* tabLayout = new QVBoxLayout();
    w->setLayout(tabLayout);

    /*route type group box*/
    QGroupBox* typeGB = new QGroupBox(tr("Type"));
    tabLayout->addWidget(typeGB);
    QVBoxLayout* typeL = new QVBoxLayout();
    typeGB->setLayout(typeL);

    typeBG = new QButtonGroup(typeGB);
    typeBG->setExclusive(true);

    QRadioButton* normalRouteRB = new QRadioButton(tr("&Normal route"));
    typeL->addWidget(normalRouteRB);
    typeBG->addButton(normalRouteRB,0 );

    QRadioButton* detourRouteRB = new QRadioButton(tr("&Detour route"));
    typeL->addWidget(detourRouteRB);
    typeBG->addButton(detourRouteRB, 1);

    QRadioButton* helpRouteRB = new QRadioButton(tr("&Help route"));
    typeL->addWidget(helpRouteRB);
    typeBG->addButton(helpRouteRB, 2);

    QRadioButton* normalShuntingRB = new QRadioButton(tr("Normal &shunting"));
    typeL->addWidget(normalShuntingRB);
    typeBG->addButton(normalShuntingRB, 3);

    QRadioButton* detourShuntingRB = new QRadioButton(tr("Detour shuntin&g"));
    typeL->addWidget(detourShuntingRB);
    typeBG->addButton(detourShuntingRB, 4);

    // push groub box to top
    tabLayout->addStretch();

    tabWidget->addTab(w, tr("&Type"));
}

/*
 * add tab with route element list
 */
void RouteDialog::addElementsTab()
{
    QWidget *w = new QWidget();
    /*vertical layout for identificationGB and an other horiz. layout*/
    QVBoxLayout* tabLayout = new QVBoxLayout();
    w->setLayout(tabLayout);

    /*route elements group box*/
    QGroupBox* routeElementsGB = new QGroupBox(tr("Route elements"));
    tabLayout->addWidget(routeElementsGB);
    QHBoxLayout* routeElL = new QHBoxLayout();
    routeElementsGB->setLayout(routeElL);

    /* list with route elements */
    elementsLV = new QTreeWidget(routeElementsGB);
    elementsLV->setUniformRowHeights(true);
    elementsLV->setAllColumnsShowFocus(true);
    elementsLV->setRootIsDecorated(false);
    routeElL->addWidget(elementsLV);
    QTreeWidgetItem* headerItem = new QTreeWidgetItem(
            QStringList() << tr("No") << tr("Name") << tr("Bus")
            << tr("Address") << tr("State"), QTreeWidgetItem::UserType + 2);
    headerItem->setTextAlignment(0, Qt::AlignCenter);
    headerItem->setTextAlignment(2, Qt::AlignRight);
    headerItem->setTextAlignment(3, Qt::AlignRight);
    headerItem->setTextAlignment(4, Qt::AlignCenter);
    elementsLV->setHeaderItem(headerItem);
    elementsLV->header()->resizeSection(0, 50);
    elementsLV->header()->resizeSection(1, 50);
    elementsLV->header()->resizeSection(2, 40);
    elementsLV->header()->resizeSection(3, 60);
    elementsLV->header()->resizeSection(4, 50);
#if QT_VERSION >= 0x050000
    elementsLV->header()->setSectionResizeMode(QHeaderView::Interactive);
#else
    elementsLV->header()->setResizeMode(QHeaderView::Interactive);
#endif
    /*TODO: elementsLV->setColumnWidthMode(0, QTreeWidget::Maximum);*/
    connect(elementsLV, SIGNAL(currentItemChanged(QTreeWidgetItem*,
                    QTreeWidgetItem*)),
            this, SLOT(elementsLVChanged(QTreeWidgetItem*,
                    QTreeWidgetItem*)));

    /*column with Add, Remove, ... buttons*/
    QVBoxLayout* routeElBtnLayout = new QVBoxLayout();
    routeElL->addLayout(routeElBtnLayout);

    routeElBtnLayout->addStretch();

    upPB = new QPushButton(tr("&Up"));
    connect(upPB, SIGNAL(clicked()), this, SLOT(upListElement()));
    routeElBtnLayout->addWidget(upPB);
    upPB->setEnabled(false);

    downPB = new QPushButton(tr("Do&wn"));
    connect(downPB, SIGNAL(clicked()), this, SLOT(downListElement()));
    routeElBtnLayout->addWidget(downPB);
    downPB->setEnabled(false);

    editPB = new QPushButton(tr("&Edit"));
    connect(editPB, SIGNAL(clicked()), this, SLOT(editListElement()));
    routeElBtnLayout->addWidget(editPB);
    editPB->setEnabled(false);

    addPB = new QPushButton(tr("&Add"));
    connect(addPB, SIGNAL(clicked()), this, SLOT(addElementToList()));
    routeElBtnLayout->addWidget(addPB);

    removePB = new QPushButton(tr("&Remove"));
    connect(removePB, SIGNAL(clicked()), this, SLOT(removeElementFromList()));
    routeElBtnLayout->addWidget(removePB);
    removePB->setEnabled(false);

    routeElBtnLayout->addStretch();

    // push buttons to the left
    routeElL->addStretch();

    tabWidget->addTab(w, tr("&Elements"));
}


void RouteDialog::setRouteName(const QString& rname)
{
    routeNameLE->setText(rname);
}


QString RouteDialog::getRouteName()
{
    return routeNameLE->text();
}


void RouteDialog::setRouteNumber(unsigned int rnumber)
{
    routeNumberLE->setText(QString::number(rnumber));
}


unsigned int RouteDialog::getRouteNumber()
{
    return routeNumberLE->text().toUInt();
}


void RouteDialog::setRouteTrain(unsigned int train)
{
    routeTrainLE->setText(QString::number(train));
}


unsigned int RouteDialog::getRouteTrain()
{
    return routeTrainLE->text().toUInt();
}


void RouteDialog::setRouteLength(unsigned int length)
{
    routeLengthLE->setText(QString::number(length));
}


unsigned int RouteDialog::getRouteLength()
{
    return routeLengthLE->text().toUInt();
}


void RouteDialog::setRouteSpeedLimit(unsigned int limit)
{
    routeSpeedLimitLE->setText(QString::number(limit));
}


unsigned int RouteDialog::getRouteSpeedLimit()
{
    return routeSpeedLimitLE->text().toUInt();
}


//TODO section

void RouteDialog::setEntrySignalData(const stateElement& signal)
{
    // name is set by "updateEntrySignalName"
    startSignalSrcpBusLE->setText(QString::number(signal.bus));
    startSignalAddressLE->setText(QString::number(signal.address));
    startSignalStateSB->setValue(signal.state);
    startSignalElPtr = signal.elemPtr;
}

/* Information about a second element in layout with same function is
 * lost here. It will be back when layout is loaded from file.*/
void RouteDialog::getEntrySignalData(stateElement& signal)
{
    signal.name = startSignalNameLE->text();
    signal.bus = startSignalSrcpBusLE->text().toUInt();
    signal.address = startSignalAddressLE->text().toUInt();
    signal.state = startSignalStateSB->value();
    if (signal.elemPtr != startSignalElPtr) {
        signal.elemPtr = startSignalElPtr;
    }
}


void RouteDialog::setExitSignalData(const stateElement& signal)
{
    // name is set by "updateExitSignalName"
    stopSignalSrcpBusLE->setText(QString::number(signal.bus));
    stopSignalAddressLE->setText(QString::number(signal.address));
    stopSignalElPtr = signal.elemPtr;
}

/* Information about a second element in layout with same function is
 * lost here. It will be back when layout is loaded from file.*/
void RouteDialog::getExitSignalData(stateElement& signal)
{
    signal.name = stopSignalNameLE->text();
    signal.bus = stopSignalSrcpBusLE->text().toUInt();
    signal.address = stopSignalAddressLE->text().toUInt();
    if (signal.elemPtr != stopSignalElPtr) {
        signal.elemPtr = stopSignalElPtr;
    }
}


void RouteDialog::setSignalTracks(unsigned int entry, unsigned int exit)
{
    entryTrackNumberLE->setText(QString::number(entry));
    exitTrackNumberLE->setText(QString::number(exit));
}


void RouteDialog::getSignalTracks(unsigned int& entry, unsigned int& exit)
{
    entry = entryTrackNumberLE->text().toUInt();
    exit = exitTrackNumberLE->text().toUInt();
}


void RouteDialog::setActivateData(const PortState& port)
{
    activatefbCB->setChecked(port.used);
    // this should normaly not be necessary:
    activateCBchanged(port.used);
    QAbstractButton* acrtbtn = activateRouteBG->button((int)port.switchtooff);
    if (acrtbtn != NULL)
        acrtbtn->setChecked(true);
    else
        qWarning("Unexpected port state found: %d", port.switchtooff);

    activateSrcpBusLE->setText(QString::number(port.bus));
    activateContactSB->setValue(port.address);
    activateContactSBChanged(port.address);
}


void RouteDialog::getActivateData(PortState& port)
{
    port.used = activatefbCB->isChecked();
    port.switchtooff = (activateRouteBG->checkedId() == 1);
    port.bus = activateSrcpBusLE->text().toUInt();
    port.address = activateContactSB->value();
}


void RouteDialog::setReleaseData(const PortState& port)
{
    releasefbCB->setChecked(port.used);
    // this should normaly not be necessary:
    releaseCBchanged(port.used);
    QAbstractButton* relbtn = releaseRouteBG->button((int)port.switchtooff);
    if (relbtn != NULL)
        relbtn->setChecked(true);
    else
        qWarning("Unexpected port state found: %d", port.switchtooff);
    releaseSrcpBusLE->setText(QString::number(port.bus));
    releaseContactSB->setValue(port.address);
    releaseContactSBChanged(port.address);
}


void RouteDialog::getReleaseData(PortState& port)
{
    port.used = releasefbCB->isChecked();
    port.switchtooff = (releaseRouteBG->checkedId() == 1);
    port.bus = releaseSrcpBusLE->text().toUInt();
    port.address = releaseContactSB->value();
}


void RouteDialog::setRouteType(int type)
{
    typeBG->button(type)->setChecked(true);
}


int RouteDialog::getRouteType()
{
    return typeBG->checkedId();
}


void RouteDialog::setRouteElements(const QList<stateElement*>& items)
{
    //RouteElementLVI* element;
    int i = 0;

    QList<stateElement*>::const_iterator it;
    for (it = items.constBegin(); it != items.constEnd(); ++it) {
       /* element =*/ new RouteElementLVI(elementsLV, i, *it);
        ++i;
    }
}


void RouteDialog::getRouteElements(QList<stateElement*>& items)
{
/*    int i = elementsLV->childCount();*/
    int i = elementsLV->topLevelItemCount();

    if (i > 0) {
        stateElement* se;
        QTreeWidgetItemIterator it(elementsLV);

        while (*it) {
            se = new stateElement;
            if (se != NULL) {
                static_cast<RouteElementLVI*>(*it)->getStateElementData(se);
                items.append(se);
            }
            // else no memory available
            ++it;
        }
    }
}


void RouteDialog::activateCBchanged(bool isChecked)
{
    activateRouteGB->setEnabled(isChecked);
    activateSrcpBusLB->setEnabled(isChecked);
    activateSrcpBusLE->setEnabled(isChecked);
    activateContactLB->setEnabled(isChecked);
    activateContactSB->setEnabled(isChecked);
    activateModuleLB->setEnabled(isChecked);
    activateModuleLE->setEnabled(isChecked);
    activatePortLB->setEnabled(isChecked);
    activatePortLE->setEnabled(isChecked);
}


void RouteDialog::releaseCBchanged(bool isChecked)
{
    releaseRouteGB->setEnabled(isChecked);
    releaseSrcpBusLB->setEnabled(isChecked);
    releaseSrcpBusLE->setEnabled(isChecked);
    releaseContactLB->setEnabled(isChecked);
    releaseContactSB->setEnabled(isChecked);
    releaseModuleLB->setEnabled(isChecked);
    releaseModuleLE->setEnabled(isChecked);
    releasePortLB->setEnabled(isChecked);
    releasePortLE->setEnabled(isChecked);
}


void RouteDialog::activateContactSBChanged(int contact)
{
    // FB_16 = 0, FB_8 = 1
    int inputs = 16 - (pref.fbfactor * 8);
    int module = (contact - 1) / inputs + 1;
    int port = contact - (module - 1) * inputs;
    activateModuleLE->setText(QString::number(module));
    activatePortLE->setText(QString::number(port));
}


void RouteDialog::releaseContactSBChanged(int contact)
{
    int inputs = 16 - (pref.fbfactor * 8);
    int module = (contact - 1) / inputs + 1;
    int port = contact - (module - 1) * inputs;
    releaseModuleLE->setText(QString::number(module));
    releasePortLE->setText(QString::number(port));
}


void RouteDialog::elementsLVChanged(QTreeWidgetItem* lvi,
        QTreeWidgetItem* previous)
{
    if (lvi == NULL) {
        upPB->setEnabled(false);
        downPB->setEnabled(false);
	editPB->setEnabled(false);
	removePB->setEnabled(false);
    }
    else {
        upPB->setEnabled(elementsLV->itemAbove(lvi) != NULL);
        downPB->setEnabled(elementsLV->itemBelow(lvi) != NULL);
        bool haselements = (elementsLV->topLevelItemCount() > 0);
	editPB->setEnabled(haselements);
	removePB->setEnabled(haselements);
    }
}


void RouteDialog::removeElementFromList()
{
    QTreeWidgetItem* lvi = elementsLV->currentItem();
    if (lvi != NULL) {
        QTreeWidgetItem* nextlvi = elementsLV->itemBelow(lvi);
        int tli = elementsLV->indexOfTopLevelItem(lvi);
        if (tli != -1) {
            elementsLV->takeTopLevelItem(tli);
            delete lvi;
        }
        updateListIndexNumbersFrom(nextlvi);

        // update button states if list is empty
        /*if (elementsLV->childCount() == 0) {*/
        if (elementsLV->topLevelItemCount() == 0) {
            editPB->setEnabled(false);
            removePB->setEnabled(false);
        }
    }
}


void RouteDialog::startSignalBusChanged(const QString& bstr)
{
    if (!bstr.isEmpty()) {
        int bus = bstr.toInt();
        int address = startSignalAddressLE->text().toInt();
        updateEntrySignalName(bus, address);
    }
}


void RouteDialog::startSignalAddressChanged(const QString& astr)
{
    if (!astr.isEmpty()) {
        int address = astr.toInt();
        int bus = startSignalSrcpBusLE->text().toInt();
        updateEntrySignalName(bus, address);
    }
}


void RouteDialog::updateEntrySignalName(int bus, int address)
{
    element* el = NULL;
    // send signal to route, routingviewer, gbs
    emit getElementByAddress(bus, address, &el);

    startSignalElPtr = el;
    if (el == NULL)
        startSignalNameLE->setText(tr("Error"));
    else {
        startSignalNameLE->setText(el->getLabelText());
        int ac = el->getAddressCount();
        if (ac == 1) {
            if (el->hasThreeStates())
                startSignalStateSB->setMaximum(2);
            else
                startSignalStateSB->setMaximum(1);
        }
        else
            startSignalStateSB->setMaximum(3);
    }
}


void RouteDialog::stopSignalBusChanged(const QString& bstr)
{
    if (!bstr.isEmpty()) {
        int bus = bstr.toInt();
        int address = stopSignalAddressLE->text().toInt();
        updateExitSignalName(bus, address);
    }
}


void RouteDialog::stopSignalAddressChanged(const QString& astr)
{
    if (!astr.isEmpty()) {
        int address = astr.toInt();
        int bus = stopSignalSrcpBusLE->text().toInt();
        updateExitSignalName(bus, address);
    }
}


void RouteDialog::updateExitSignalName(int bus, int address)
{
    element* el = NULL;
    // send signal to route, routingviewer, gbs
    emit getElementByAddress(bus, address, &el);

    stopSignalElPtr = el;
    if (el == NULL)
        stopSignalNameLE->setText(tr("Error"));
    else
        stopSignalNameLE->setText(el->getLabelText());
}


void RouteDialog::upListElement()
{
    QTreeWidgetItem* lvi = elementsLV->currentItem();
    if (lvi != NULL) {
        int idx = lvi->text(0).toInt();
        lvi->setText(0, QString::number(idx - 1));
        QTreeWidgetItem* ulvi = elementsLV->itemAbove(lvi);
        if (ulvi != NULL)
            ulvi->setText(0, QString::number(idx));
        elementsLV->sortItems(0, Qt::AscendingOrder);
        elementsLVChanged(lvi, NULL);
    }
}


void RouteDialog::downListElement()
{
    QTreeWidgetItem* lvi = elementsLV->currentItem();
    if (lvi != NULL) {
        int idx = lvi->text(0).toInt();
        lvi->setText(0, QString::number(idx + 1));
        QTreeWidgetItem* dlvi = elementsLV->itemBelow(lvi);
        if (dlvi != NULL)
            dlvi->setText(0, QString::number(idx));
        elementsLV->sortItems(0, Qt::AscendingOrder);
        elementsLVChanged(lvi, NULL);
    }
}


void RouteDialog::editListElement()
{
    RouteElementLVI* relvi = (RouteElementLVI*) elementsLV->currentItem();
    if (relvi == NULL)
        return;

    RouteElementDialog* rteDlg = new RouteElementDialog(this);

    if (rteDlg == NULL)
        return;

    connect(rteDlg, SIGNAL(getElementByAddress(const int, const int,
                    element**)), this,
            SIGNAL(getElementByAddress(const int, const int,
                    element**)));

    stateElement* se = new stateElement;

    if (se != NULL) {
        relvi->getStateElementData(se);
        rteDlg->setStateElementData(se);

        if (rteDlg->exec() == QDialog::Accepted) {
            rteDlg->getStateElementData(se);
            relvi->setStateElementData(se);
        }
        delete se;
    }

    disconnect(rteDlg, SIGNAL(getElementByAddress(const int, const int,
                    element**)), this,
            SIGNAL(getElementByAddress(const int, const int,
                    element**)));
    delete rteDlg;
}


void RouteDialog::addElementToList()
{
    int ec = elementsLV->topLevelItemCount();
    ++ec;

    RouteElementDialog* rteDlg = new RouteElementDialog(this);

    if (rteDlg == NULL)
        return;

    rteDlg->setWindowTitle(tr("Add new element"));

    connect(rteDlg, SIGNAL(getElementByAddress(const int, const int,
                    element**)), this,
            SIGNAL(getElementByAddress(const int, const int,
                    element**)));

    stateElement* se = new stateElement;

    if (se != NULL) {
        se->name = "";
        se->bus = 1;
        se->address = 0;
        se->state = 0;
        se->elemPtr = NULL;
        rteDlg->setStateElementData(se);

        if (rteDlg->exec() == QDialog::Accepted) {
            rteDlg->getStateElementData(se);
            RouteElementLVI* relvi = new RouteElementLVI(elementsLV,
                    ec , se);
            if (relvi != NULL) {
                relvi->setStateElementData(se);
                //TODO: listview (string) sorting fails if items count > 9
                elementsLV->sortItems(0, Qt::AscendingOrder);
                // set focus to new element
                relvi->setSelected(true);
                // update button states
                elementsLVChanged(relvi, NULL);
            }
        }
        delete se;
    }

    disconnect(rteDlg, SIGNAL(getElementByAddress(const int, const int,
                    element**)), this,
            SIGNAL(getElementByAddress(const int, const int,
                    element**)));
    delete rteDlg;
}


void RouteDialog::updateListIndexNumbersFrom(QTreeWidgetItem* lvi)
{
    if (lvi == NULL)
        return;

    QTreeWidgetItem* newlvi = lvi;
    while (newlvi != NULL) {
        int idx = newlvi->text(0).toInt();
        --idx;
        newlvi->setText(0, QString::number(idx));
        QTreeWidgetItem* nextlvi = elementsLV->itemBelow(newlvi);
        newlvi = nextlvi;
    }
}

