/***************************************************************************
                           variantdialog.cpp
                           -------------------------------
    copyright            : (C) 2010 Guido Scholz
    e-mail               : guido.scholz@bayernline.de
    last modified        : $Date: 2009/11/01 20:40:38 $
                           $Revision: 1.82 $
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/***************************************************************************
 This file provides an user interface to select an element variant
 ***************************************************************************/

#include <QLabel>
#include <QLayout>
#include <QDialogButtonBox>

#include "variantdialog.h"


VariantDialog::VariantDialog(QWidget* parent): QDialog(parent),
    variantid(0)
{
    setObjectName("VariantDialog");
    setWindowTitle(tr("Edit variant"));

    /*Layout to separate OK Cancel buttons from the upper rest*/
    QBoxLayout* baseLayout = new QVBoxLayout(this);
    baseLayout->setSpacing(12);

    /*variant display*/
    imageLabel = new QLabel(this);
    imageLabel->setAlignment(Qt::AlignHCenter);
    baseLayout->addWidget(imageLabel);

    /*variant selection*/
    variantGB = new QGroupBox(tr("Variant"), this);
    baseLayout->addWidget(variantGB);

    QVBoxLayout* variantGBL = new QVBoxLayout();
    variantGB->setLayout(variantGBL);

    variantBG = new QButtonGroup(variantGBL);
    variantBG->setExclusive(true);
    variantBG->setObjectName("variantBG");

    connect(variantBG, SIGNAL(buttonClicked(int)), this,
            SLOT(selectionChanged(int)));

    QDialogButtonBox* buttonBox = new QDialogButtonBox(
            QDialogButtonBox::Ok |
            QDialogButtonBox::Cancel);

    connect(buttonBox, SIGNAL(accepted()), this, SLOT(accept()));
    connect(buttonBox, SIGNAL(rejected()), this,
            SLOT(reject()));
    baseLayout->addWidget(buttonBox);
}


void VariantDialog::setChoice(int choice)
{
    QAbstractButton *button;
    button = variantBG->button(choice);
    if (button != NULL) {
        button->setChecked(true);
        selectionChanged(choice);
    }
}


int VariantDialog::getChoice()
{
    return variantBG->checkedId();
}


void VariantDialog::addVariant(const QString& text)
{
    QRadioButton* button = new QRadioButton(text, variantGB);
    variantBG->addButton(button);
    variantBG->setId(button, variantid);
    variantid++;
    variantGB->layout()->addWidget(button);
}


void VariantDialog::addVariant(const QString& text, const QPixmap& pm)
{
    addVariant(text);
    pmList.append(pm);
}


void VariantDialog::selectionChanged(int index)
{
    if (index >= 0 && index < pmList.count())
        imageLabel->setPixmap(pmList[index]);
}


/*hide/show image label*/
void VariantDialog::showImage(bool show)
{
    if (show)
        imageLabel->show();
    else
        imageLabel->hide();
}
